/* AMX Mod X
*	Snake vs Monkey.
*
* This plugin is licenced under the AGPLv3 Licence
* 
* The AGPL license differs from the other GNU licenses in 
* that it was built for network software. You can distribute 
* modified versions if you keep track of the changes and the 
* date you made them. As per usual with GNU licenses, you must 
* license derivatives under AGPL. 
* It provides the same restrictions and freedoms as the GPLv3 
* but with an additional clause which makes it so that source code 
* must be distributed along with web publication. 
* Since web sites and services are never distributed in the 
* traditional sense, the AGPL is the GPL of the web.
*
* This file is provided as is (no warranties)
*/


#include <amxmodx>
#include <amxmisc>
#include <fakemeta>
#include <engine>
#include <fun>
#include <time>
#include <hlstocks>
#include <hamsandwich>
#include <hl_wpnmod>
#if AMXX_VERSION_NUM < 183
    #include <dhudmessage>
#endif  

#define PLUGIN_NAME "Snake vs. Monkey"
#define PLUGIN_VERSION "0.8.9 beta"
#define PLUGIN_AUTHOR "Gabe Iggy"

#define SNAKE_MODEL "Snake"
#define MONKEY_MODEL "Monkey"
//#define FOX_MODEL "CyborgNinja"

#define KEROTAN_MODEL "models/svm/kerotan.mdl"
#define KEROTAN_ENTITY "monster_generic"
#define KEROTAN_SOUND "svm/kerotan_hit.wav"
#define KEROTAN_SPRITE "sprites/kerotan.spr"

#define SNAKE_PAINSOUND_1 "svm/snake_pain1.wav"
#define SNAKE_PAINSOUND_2 "svm/snake_pain2.wav"
#define SNAKE_KILLED_SOUND "svm/snake_die.wav"

#define MONKEY_PAINSOUND "svm/monkey_vo_pain1.wav"
#define MONKEY_KILLED_SOUND_1 "svm/monkey_vo_die1.wav"
#define MONKEY_KILLED_SOUND_2 "svm/monkey_vo_die3.wav"

//#define MONKEY_COMMS_ALERT_1 "svm/monkey_vo_alert1.wav"
//#define MONKEY_COMMS_ALERT_2 "svm/monkey_vo_alert2.wav"
//#define MONKEY_COMMS_FOOTPRINTS "svm/monkey_vo_footprints.wav"
//#define MONKEY_COMMS_SOUND_1 "svm/monkey_vo_sound1.wav"
//#define MONKEY_COMMS_SOUND_2 "svm/monkey_vo_sound2.wav"

#define ROUND_START "sound/svm/svm_roundstart.mp3"
#define ROUND_END_SNAKE "sound/svm/snake_wins.mp3"
#define ROUND_END_MONKEYS "sound/svm/snake_loses.mp3"

#define ROUND_MUSIC "sound/svm/svm_game.mp3"
#define ROUND_MUSIC_DURATION 115
#define ROUND_MUSIC_TASK 35987

#define END_MAP_SOUND "svm/intermission.wav"

#define SNAKE_SPAWN_ENTNAME "info_target"

//#define MAX_SNAKE_SPAWN_POINTS 2 //Todo, allow dynamic snake spawn
#define MAX_KEROTANS 2

#define TASK_SENDVICTIMTOSPEC 62746
#define TASK_ROUND_CHECK 37746
#define TASK_FREEZE_TIME 57541
#define TASK_ROUND_TIME 3245
#define TASK_SPAWN_POST 24755
#define TASK_ROUND_END 41321
#define TASK_FIRST_JOIN 48799
#define TASK_UNFREEZE 98742

#define MAX_ENTITIES 2048

new const g_aszClearFieldEntsClass[][] = {
	"bolt",
	"monster_snark",
	"monster_satchel",
	"monster_tripmine",
	"beam", // this removes beam of tripmine
	"weaponbox"
};

new cvar_svmEnabled, cvar_roundFreezeTime, cvar_roundTime;
new g_szOptimalTeam[32] = "";
new g_iPlayerQueue[33] = 0;
new g_iPlayerTeam[33] = 0;
new g_iFirstSpawn[33] = 0;
new g_iSpecialEnts[2048];
new g_iKerotans[MAX_KEROTANS];
new g_bRoundStarted = 0;
new g_iMonkeyCount = 0;
new g_iCurrentSnake = 0;
new g_iRoundTime = 0;
new g_iRoundFreezeTime = 0;
new g_iKeroExists = 0;
new g_iKeroSpriteIndex = 0;

//saving
new filename[128];

/** 
 *      PLUGIN INFO
**/
public plugin_init()
{
    register_plugin(PLUGIN_NAME, PLUGIN_VERSION, PLUGIN_AUTHOR);
    register_dictionary("time.txt");

    cvar_svmEnabled = register_cvar("amx_svm_enable","1");
    cvar_roundTime = register_cvar("amx_svm_round_time","200");
    cvar_roundFreezeTime = register_cvar("amx_svm_round_freeze_time","10");
}

public plugin_precache()
{
    new model[128];
    formatex(model, charsmax(model), "models/player/%s/%s.mdl", SNAKE_MODEL, SNAKE_MODEL);
    precache_model(model);
    formatex(model, charsmax(model), "models/player/%s/%s.mdl", MONKEY_MODEL, MONKEY_MODEL);
    precache_model(model);

    precache_model(KEROTAN_MODEL);
    precache_sound(KEROTAN_SOUND);

    precache_sound(SNAKE_PAINSOUND_1);
    precache_sound(SNAKE_PAINSOUND_2);
    precache_sound(SNAKE_KILLED_SOUND);

    precache_sound(MONKEY_KILLED_SOUND_1);
    precache_sound(MONKEY_KILLED_SOUND_2);
    precache_sound(MONKEY_PAINSOUND);

    precache_sound(END_MAP_SOUND);

    g_iKeroSpriteIndex = precache_model(KEROTAN_SPRITE);
    precache_generic(ROUND_START);
    precache_generic(ROUND_MUSIC);
    precache_generic(ROUND_END_SNAKE);
    precache_generic(ROUND_END_MONKEYS);
}

public plugin_cfg()
{
    if(get_pcvar_num(cvar_svmEnabled))
	{
        if(get_cvar_num("mp_welcomecam"))
        {
            server_print("ERROR: Bugfixed HLDM's mp_welcomecam enabled! This gamemode requires this CVAR to be off!");
            set_fail_state("mp_welcomecam enabled");
            return;
        }

        if(!get_cvar_num("mp_teamplay"))
        {
            server_print("ERROR: mp_teamplay not enabled. The gamemode requires Team Play to work.");
            set_fail_state("mp_teamplay not enabled");
            return;
        }

        if(!get_cvar_num("mp_allowmonsters"))
        {
            server_print("ERROR: mp_allowmonsters not enabled. The kerotan can't spawn otherwise.");
            set_fail_state("mp_allowmonsters not enabled");
            return;
        }

        if(!is_plugin_loaded("Stealth Box"))
        {
            server_print("ERROR: Stealth Box is not enabled. This gamemode depends on this plugin.");
            set_fail_state("box_philosophy.amxx not loaded");
            return;
        }

        new buff[32];
        get_cvar_string("mp_teamlist", buff, charsmax(buff));
        formatex(g_szOptimalTeam, charsmax(g_szOptimalTeam), "%s;%s", SNAKE_MODEL, MONKEY_MODEL);
        if(!equali(buff, g_szOptimalTeam))
        {
            server_print("ERROR: mp_teamlist is not '%s;%s'. The gamemode requires this teamlist to work.", SNAKE_MODEL, MONKEY_MODEL);
            set_fail_state("mp_teamlist is not '%s;%s'", SNAKE_MODEL, MONKEY_MODEL);
            return;
        }

        if(!get_cvar_num("mp_falldamage"))
            server_print("WARNING: mp_falldamage not enabled. This will make the game less risky, and thus, less shenanigans will occur :)");

        if(!get_cvar_num("mp_friendlyfire"))
            server_print("WARNING: mp_friendlyfire not enabled. This will make the game less risky, and thus, less shenanigans will occur :)");

        register_forward(FM_GetGameDescription, "game_description"); 

        register_message(SVC_INTERMISSION, "map_end");

        RegisterHam(Ham_Killed, "player", "player_killed_post", 1);     
        RegisterHam(Ham_Spawn, "player", "player_spawn_pre");           
        RegisterHam(Ham_Spawn, "player", "player_spawn_post", 1);      
        RegisterHam(Ham_TakeDamage, "player", "player_take_damage_pre");      
        RegisterHam(Ham_ChangeYaw, "monster_generic", "dont_turn");        
        RegisterHam(Ham_TakeDamage, "monster_generic", "damage_kerotan");  
        RegisterHam(Ham_Touch, "monster_generic", "reach_kerotan");   
        RegisterHam(Ham_BloodColor, "monster_generic", "bloodcolor_kerotan"); 

        register_clcmd("spectate", "cmd_spectate");  
        register_clcmd("jointeam", "cmd_join_team");       
        register_clcmd("changeteam", "cmd_change_team");
        register_clcmd("create_kerotan", "spawn_kerotan");
        register_clcmd("create_snake_spawn", "spawn_snake_spawnpoint");
        register_clcmd("svm_save_ents", "save_ents");

        new mapname[32];
        get_mapname(mapname, charsmax(mapname));
        get_filename(mapname);
        load_ents(filename);
        new found = 0;
        new ent = -1;
        while ((ent = find_ent_by_class(ent, KEROTAN_ENTITY)))
        {
            if(g_iSpecialEnts[ent] == 1)
            {
                found = 1;
                break;
            }
        }
        
        if(!found)
            server_print("WARNING: The kerotan was not found in the map. Snake can win a round only by killing everyone else.");
        
        found = 0;
        while ((ent = find_ent_by_class(ent, SNAKE_SPAWN_ENTNAME)))
        {
            if(g_iSpecialEnts[ent] == 2)
            {
                found = 1;
                break;
            }
        }

        if(!found)
            server_print("WARNING: Snake's spawnpoint is missing from the map. Snake will use any regular spawn.");

        remove_hldm_entities();

        set_task(1.0, "round_checker", TASK_ROUND_CHECK, "", 0, "b");
    }
    else
		pause("ad");
}

/** 
 *      GAME FORWARDS
**/

public client_disconnected(id)
{
    g_iPlayerTeam[id] = 0;
    g_iFirstSpawn[id] = 0;

    if(!task_exists(TASK_ROUND_END) && !task_exists(TASK_FREEZE_TIME))
    {
        if(g_iPlayerTeam[id] == 2)
            g_iMonkeyCount--;

        if(g_iMonkeyCount == 0)
            finish_round(2);
        else if(g_iCurrentSnake == id)
            finish_round(1);
    }
}

/** 
 *      GAME HOOKS
**/

public map_end()
{
    new iPlayers[32], iPnum;
    get_players(iPlayers, iPnum);

    for(new i = 0; i < iPnum; i++)
    {
        client_cmd(iPlayers[i], "mp3 stop");
        client_cmd(iPlayers[i], "spk %s", END_MAP_SOUND);
    }
}

public game_description() 
{ 
    new gamename[32];
    formatex(gamename, charsmax(gamename), "%s v%s", PLUGIN_NAME, PLUGIN_VERSION);
    forward_return(FMV_STRING, gamename); 
    return FMRES_SUPERCEDE; 
}

public player_killed_post(id)
{
    g_iPlayerTeam[id] = 0;

    if(id == g_iCurrentSnake)
        emit_sound(id, CHAN_VOICE, SNAKE_KILLED_SOUND, VOL_NORM, ATTN_NONE, SND_SPAWNING, PITCH_NORM);
    else
    {
        if(random_num(0, 1))
            emit_sound(id, CHAN_VOICE, MONKEY_KILLED_SOUND_1, VOL_NORM, ATTN_NORM, SND_SPAWNING, PITCH_NORM);
        else  
            emit_sound(id, CHAN_VOICE, MONKEY_KILLED_SOUND_2, VOL_NORM, ATTN_NORM, SND_SPAWNING, PITCH_NORM);
    }

    if(!task_exists(TASK_ROUND_END) && !task_exists(TASK_FREEZE_TIME))
    {
        new victimModel[32];
        hl_get_user_model(id, victimModel, charsmax(victimModel));
        if(equali(MONKEY_MODEL, victimModel))
            g_iMonkeyCount--;

        if(g_iMonkeyCount == 0 || !is_user_alive(g_iCurrentSnake))
        {
            if(g_iMonkeyCount == 0)
                finish_round(2);
            else 
                finish_round(1);
        }
    }

    if (g_bRoundStarted)
		set_task(5.0, "send_player_to_spec", id + TASK_SENDVICTIMTOSPEC);
}

public player_spawn_post(id)
{
    if(!g_iFirstSpawn[id])
    {
        g_iFirstSpawn[id] = 1;
        if(g_bRoundStarted)
        {
            set_task(0.1, "player_send_to_spec_on_join", TASK_FIRST_JOIN+id);
            return;
        }
    }

    strip_user_weapons(id);
    set_task(0.1, "player_spawn_post_after", TASK_SPAWN_POST+id)
}

public player_spawn_pre(id)
{
    if(!g_iFirstSpawn[id])
        return HAM_IGNORED;

    if(task_exists(TASK_ROUND_TIME))
        return HAM_SUPERCEDE;
    
    return HAM_IGNORED;
}

public damage_kerotan(id)
{
    if(g_iSpecialEnts[id] == 1)
    {
        emit_sound(id, CHAN_VOICE, KEROTAN_SOUND, VOL_NORM, ATTN_NORM, SND_SPAWNING, PITCH_NORM);
        return HAM_SUPERCEDE;
    }
    else
        return HAM_IGNORED;
}

public player_take_damage_pre(id)
{
    if(task_exists(TASK_ROUND_END))
        return HAM_SUPERCEDE;

    if(id == g_iCurrentSnake)
    {
        if(random_num(0, 1))
            emit_sound(id, CHAN_VOICE, SNAKE_PAINSOUND_1, VOL_NORM, ATTN_NORM, SND_SPAWNING, PITCH_NORM);
        else  
            emit_sound(id, CHAN_VOICE, SNAKE_PAINSOUND_2, VOL_NORM, ATTN_NORM, SND_SPAWNING, PITCH_NORM);
    }
    else
    {
        emit_sound(id, CHAN_VOICE, MONKEY_PAINSOUND, VOL_NORM, ATTN_NORM, SND_SPAWNING, PITCH_NORM);
    }

    return HAM_HANDLED;
}

public reach_kerotan(id, idother)
{
    if(g_iSpecialEnts[id] == 1 && (0 < idother <= get_maxplayers()))
    {
        if(!task_exists(TASK_ROUND_END))
        {
            new otherModel[32];
            hl_get_user_model(idother, otherModel, charsmax(otherModel));
            if(equali(SNAKE_MODEL, otherModel))
            {
                finish_round(2);
                return HAM_HANDLED;
            }
        }
    }
    
    return HAM_IGNORED;
}

public dont_turn(id)
{
    if(g_iSpecialEnts[id] == 1)
    {
	    return HAM_SUPERCEDE;
    }
    else
        return HAM_IGNORED;
}

public bloodcolor_kerotan(id)
{
    if(g_iSpecialEnts[id] == 1)
    {
        SetHamReturnInteger(-1);
        return HAM_OVERRIDE;
    }
    else
        return HAM_IGNORED;
}
/** 
 *      USER COMMANDS
**/

public cmd_spectate(id) 
{
	client_print(id, print_console, "* You can't use this command.");
	return PLUGIN_HANDLED;
}

public cmd_change_team(id) 
{
	client_print(id, print_console, "* You can't use this command.");
	return PLUGIN_HANDLED;
}

public cmd_join_team(id) 
{
	client_print(id, print_console, "* You can't use this command.");
	return PLUGIN_HANDLED;
}

public save_ents(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED;
	
	static file, ent, cnt, Float:origin[3], Float:angles[3];
	if ((file = fopen(filename, "w")))
	{
        fputs(file, "; Snake vs Monkey entities^n; Format: [snake|kerotan] [origin (x/y/z)] [angles (x/y/z)]");
        cnt = ent = 0;
        while ((ent = find_ent_by_class(ent, KEROTAN_ENTITY)))
        {
            if(g_iSpecialEnts[ent] == 1)
            {
                entity_get_vector(ent, EV_VEC_origin, origin);
                entity_get_vector(ent, EV_VEC_angles, angles);
                fprintf(file, "^nkerotan %f %f %f %f %f %f", origin[0], origin[1], origin[2], angles[0], angles[1], angles[2]);
                cnt++;
            }
        }

        while ((ent = find_ent_by_class(ent, SNAKE_SPAWN_ENTNAME)))
        {
            if(g_iSpecialEnts[ent] == 2)
            {
                entity_get_vector(ent, EV_VEC_origin, origin);
                entity_get_vector(ent, EV_VEC_angles, angles);
                fprintf(file, "^nsnake %f %f %f %f %f %f", origin[0], origin[1], origin[2], angles[0], angles[1], angles[2]);
                cnt++;
            }
        }
        server_print("[SNAKE VS MONKEY] %i elements saved in ^"%s^"", cnt, filename);
        client_print(id, print_console, "%i elements saved in ^"%s^"", cnt, filename);
        fclose(file);
        return 1;
	}
	else 
    {
        log_amx("[SNAKE VS MONKEY] ERROR: Can't Open Save File for Writing.");
        client_print(id, print_console, "ERROR: Can't Open Save File for Writing.");
    }
	return 0;
}

// 1- kerotan
// 2- snake spawnpoint
public spawn_system_entity(type, Float:origin[3], Float:angles[3])
{
    new entid;
    switch(type)
    {
        case 1:
        {
            entid = create_entity(KEROTAN_ENTITY);
            set_pev(entid, pev_model, KEROTAN_MODEL);
            g_iKeroExists = 1;
            g_iSpecialEnts[entid] = 1;
            g_iKerotans = push_element_to_array(g_iKerotans, entid);
        }
        case 2:
        {
            entid = create_entity(SNAKE_SPAWN_ENTNAME);
            g_iSpecialEnts[entid] = 2;
        }
    }

    angles[0] = 0.0;
    set_pev(entid, pev_origin, origin);
    set_pev(entid, pev_angles, angles);
    set_pev(entid, pev_flags, FL_NOTARGET);

    DispatchSpawn(entid);
}

public spawn_kerotan(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED;
	
	new origin[3], angles[3];
	
	pev(id, pev_origin, origin);
	pev(id, pev_angles, angles);
	
	spawn_system_entity(1, Float:origin, Float:angles);
		
	return PLUGIN_HANDLED;
}

public spawn_snake_spawnpoint(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED;
	
	new origin[3], angles[3];
	
	pev(id, pev_origin, origin);
	pev(id, pev_angles, angles);
	
	spawn_system_entity(2, Float:origin, Float:angles);
		
	return PLUGIN_HANDLED;
}

/** 
 *      TASKS
**/

public music_ingame()
{
    music_play(ROUND_MUSIC);
    set_task(Float:float(ROUND_MUSIC_DURATION), "music_ingame", ROUND_MUSIC_TASK);
}

public player_send_to_spec_on_join(taskid)
{
    new id = taskid-TASK_FIRST_JOIN;
    if(!task_exists(TASK_FREEZE_TIME))
        hl_set_user_spectator(id, true);
}

public player_spawn_post_after(taskid)
{  
    new id = taskid-TASK_SPAWN_POST;
    if(is_user_alive(id))
    {
        set_pev(id, pev_maxspeed, -1.0);
        strip_user_weapons(id); //needs to execute strip user weapons twice to prevent the server from giving you a ghost crowbar
        if(g_iCurrentSnake != id)
        {
            hl_set_user_model(id, MONKEY_MODEL);
            hl_set_user_team(id, MONKEY_MODEL);

            g_iPlayerTeam[id] = 2;
            give_item(id, "ammo_9mmbox");
            give_item(id, "ammo_9mmbox");
            give_item(id, "ammo_buckshot");
            give_item(id, "ammo_buckshot");
            give_item(id, "ammo_buckshot");
            // give_item(id, "ammo_crossbow");
            // give_item(id, "ammo_crossbow");
            // give_item(id, "ammo_crossbow");
            // give_item(id, "weapon_crossbow");
            give_item(id, "weapon_9mmAR");
            give_item(id, "weapon_tripmine");
            give_item(id, "weapon_tripmine");
            give_item(id, "weapon_tripmine");
            give_item(id, "weapon_shotgun");
            give_item(id, "weapon_glock");
        }
        else
        {
            hl_set_user_model(id, SNAKE_MODEL);
            hl_set_user_team(id, SNAKE_MODEL);
            g_iPlayerTeam[id] = 1;
            new Float:origin[3];
            new ent = -1;
            new found = 0;
            while((ent = find_ent_by_class(ent,SNAKE_SPAWN_ENTNAME)) != 0)
            {
                if(g_iSpecialEnts[ent] != 2)
                    continue;

                pev(ent, pev_origin, origin);
                found = 1;
            }
            
            //hack for Weaponmod
            new box = wpnmod_create_item("weapon_box", Float:{4096.0, 4096.0, 4096.0});
            ExecuteHamB(Ham_AddPlayerItem, id, box);
            ExecuteHamB(Ham_Item_AttachToPlayer, id, box);

            give_item(id,"ammo_357");
            give_item(id,"ammo_357");
            give_item(id,"ammo_357");
            give_item(id, "weapon_crowbar");
            give_item(id, "weapon_satchel");
            give_item(id, "weapon_satchel");
            give_item(id, "weapon_satchel");
            give_item(id, "weapon_handgrenade");
            give_item(id, "weapon_357");

            if(found)
                set_pev(id, pev_origin, origin);
        }
    }
}

public round_checker(taskid)
{
    if(get_playersnum() < 2)
    {
        if(g_bRoundStarted)
            g_bRoundStarted = 0;

        remove_task(TASK_SENDVICTIMTOSPEC);
        remove_task(TASK_FREEZE_TIME);
        remove_task(TASK_ROUND_TIME);
        remove_task(TASK_SPAWN_POST);
        remove_task(TASK_ROUND_END);
        remove_task(TASK_FIRST_JOIN);
        remove_task(ROUND_MUSIC_TASK);

        client_print(0, print_center, "Not Enough Players");
        
        new iPlayers[32], iPnum;
        get_players(iPlayers, iPnum);

        for(new i = 0; i < iPnum; i++)
        {
            if(g_iFirstSpawn[iPlayers[i]] && !hl_get_user_spectator(iPlayers[i]))
            {
                if (is_user_connected(iPlayers[i]) && is_user_alive(iPlayers[i]))
		            hl_set_user_spectator(iPlayers[i], true);
            }
        }  
    }
    else 
    {
        if(!task_exists(TASK_ROUND_END))
        {
            if(g_bRoundStarted)
            {
                g_iRoundTime -= 1;
                new iPlayers[32], iPnum;
                get_players(iPlayers, iPnum);

                new hud[125]
                new m = g_iRoundTime/60
                new h = m/60
                new s = g_iRoundTime-m*60
                m = m-h*60
                if(h)
                    formatex(hud,charsmax(hud),"%d:%02d:%02d",h,m,s)
                else
                    formatex(hud,charsmax(hud),"%d:%02d",m,s)

                for(new i = 0; i < iPnum; i++)
                {
                    if(is_user_connected(iPlayers[i]))
                    {
                        set_hudmessage(255, 255, 255, -1.0, 0.02, 0, 5.0, 5.0, 0.0, 0.0, 4)
                        show_hudmessage(iPlayers[i],hud);
                    }
                }
            }
            else
                g_iRoundFreezeTime -= 1;
        }
        
        if(!g_bRoundStarted && !task_exists(TASK_FREEZE_TIME))
            round_pre_start();

        new iPlayers[32], iPnum;
        get_players(iPlayers, iPnum);

        for(new i = 0; i < iPnum; i++)
        {
            if(is_user_connected(iPlayers[i]) && is_user_alive(iPlayers[i]))
            {
                new origin[3], Float:f_origin[3], Float:kerorigin[3], Float:endpoint[3];
                get_user_origin(iPlayers[i], origin, 1);
                IVecFVec(origin, f_origin);
                for(new x = 0;x < sizeof(g_iKerotans);x++)
                {
                    if(g_iKerotans[x])
                    {
                        pev(g_iKerotans[x], pev_origin, kerorigin);
                        trace_line(iPlayers[i], f_origin, kerorigin, endpoint);
                        message_begin(MSG_ONE_UNRELIABLE, SVC_TEMPENTITY, .player = iPlayers[i]);
                        write_byte(TE_SPRITE);
                        write_coord(floatround(endpoint[0]));
                        write_coord(floatround(endpoint[1]));
                        write_coord(floatround(endpoint[2]));
                        write_short(g_iKeroSpriteIndex);
                        write_byte(5);
                        write_byte(50);
                        message_end();
                    }
                }
            }
        }
    }
}

public round_post_start()
{
    music_ingame();
    new iPlayers[32], iPnum;
    get_players(iPlayers, iPnum);
    for(new i = 0; i < iPnum; i++)
    {
        if(!is_user_alive(iPlayers[i]))
            ExecuteHamB(Ham_Spawn, iPlayers[i]);
        if(g_iCurrentSnake != iPlayers[i])
            g_iMonkeyCount++;

        //hack to prevent frozen players when a player latejoined 
        //and spawned just when the round started
        set_task(0.2, "unfreeze_player", TASK_UNFREEZE+iPlayers[i]);
    }
    g_bRoundStarted = 1;

    g_iRoundTime = get_pcvar_num(cvar_roundTime);
    set_task(Float:float(get_pcvar_num(cvar_roundTime)), "round_time_out", TASK_ROUND_TIME);
}

public unfreeze_player(taskid)
{
    new id = taskid-TASK_UNFREEZE;
    set_pev(id, pev_maxspeed, Float:float(get_cvar_num("sv_maxspeed")));
}

public round_end(taskid)
{
    g_bRoundStarted = 0;
    g_iMonkeyCount = 0;
    g_iCurrentSnake = 0;
    round_pre_start();
}

public send_player_to_spec(taskid) 
{
	new id = taskid - TASK_SENDVICTIMTOSPEC;
	if (is_user_connected(id)) 
    {
		if (!is_user_alive(id) || is_user_bot(id)) 
			hl_set_user_spectator(id, true);
	}
}

public round_time_out(taskid)
{
    if(!task_exists(TASK_ROUND_END))
        finish_round(1);
}

/* 
 *      METHODS
 */

public finish_round(wincond)
{
    remove_task(ROUND_MUSIC_TASK);
    switch(wincond)
    {
        case 1:
        {
            new iPlayers[32], iPnum;
            get_players(iPlayers, iPnum);
            for(new i = 0; i < iPnum; i++)
            {
                new model[32];
                hl_get_user_model(iPlayers[i], model, charsmax(model));
                if(equali(model, MONKEY_MODEL))
                {
                    if(is_user_connected(iPlayers[i]))
                        set_user_frags(iPlayers[i], get_user_frags(iPlayers[i])+10);
                }
            }
            music_play(ROUND_END_MONKEYS);
            client_print(0, print_center, "MONKEYS WIN");
        }

        case 2:
        {
            if(is_user_connected(g_iCurrentSnake))
                set_user_frags(g_iCurrentSnake, get_user_frags(g_iCurrentSnake)+10);
            music_play(ROUND_END_SNAKE);
            client_print(0, print_center, "SNAKE WINS");
        }

        default:
        {
            client_print(0, print_center, "ROUND DRAW");
        }
    }

    set_task(8.0, "round_end", TASK_ROUND_END);
}

public round_pre_start()
{
    clear_corpses();
    clear_field();
    remove_task(TASK_ROUND_END);
    remove_task(TASK_ROUND_TIME);
    g_iMonkeyCount = 0;
    new iPlayers[32], iPnum;
    get_players(iPlayers, iPnum);
    new maxQueue = 0;

    music_play(ROUND_START);

    //loop to define who's the next snake
    for(new i = 0; i < iPnum; i++)
    {
        if(maxQueue <= g_iPlayerQueue[iPlayers[i]])
        {
            maxQueue = g_iPlayerQueue[iPlayers[i]];
            if(!is_user_bot(iPlayers[i]))
                g_iCurrentSnake = iPlayers[i];
        }
    }

    for(new i = 0; i < iPnum; i++)
    {
        if(hl_get_user_spectator(iPlayers[i]))
            hl_set_user_spectator(iPlayers[i], bool:false);

        ExecuteHamB(Ham_Spawn, iPlayers[i]);
        
        if(iPlayers[i] == g_iCurrentSnake)
        {
            g_iPlayerQueue[iPlayers[i]] = 0;

            //set_dhudmessage( red = 0, green = 160, blue = 0, Float:x = -1.0, Float:y = 0.65, effects = 2, Float:fxtime = 6.0, Float:holdtime = 3.0, Float:fadeintime = 0.1, Float:fadeouttime = 1.5, bool:reliable = false )
            //show_dhudmessage( index, const message[], any:... )
            set_dhudmessage(42, 127, 255, -1.0, -1.0, 2, 6.0, 10.0, 0.1, 0.2);
            //set_hudmessage(42, 127, 255, -1.0, -1.0, 0, 6.0, 12.0);
            if(g_iKeroExists)
                show_dhudmessage(iPlayers[i], "SNAKE ROUND^n^nREACH THE KEROTAN OR^nDEFEAT THE MONKEYS^nTO WIN THE MATCH");
            else   
                show_dhudmessage(iPlayers[i], "SNAKE ROUND^n^nDEFEAT THE MONKEYS^nTO WIN THE MATCH");
            
        }
        else
        {
            if(!is_user_bot(iPlayers[i])) //bots should never be snake. Sorry =W=''
                g_iPlayerQueue[iPlayers[i]] += 10;
            client_print(iPlayers[i], print_chat, "Your current Snake queue score is %i", g_iPlayerQueue[iPlayers[i]]);
            set_dhudmessage(255, 0, 0, -1.0, -1.0, 2, 6.0, 10.0, 0.1, 0.2);
            if(g_iKeroExists)
                show_dhudmessage(iPlayers[i], "MONKEY ROUND^n^nKILL SNAKE OR^nPREVENT HIM FROM^nREACHING THE KEROTAN");
            else   
                show_dhudmessage(iPlayers[i], "MONKEY ROUND^n^nFIND AND KILL^nSNAKE AT ALL COSTS");
        }
    }  
    g_iRoundFreezeTime = get_pcvar_num(cvar_roundFreezeTime);
    set_task(Float:float(get_pcvar_num(cvar_roundFreezeTime)), "round_post_start", TASK_FREEZE_TIME);
}

public remove_hldm_entities()
{
    for(new x = -1; x <= MAX_ENTITIES;x++)
    {
        if(!is_valid_ent(x))
            continue;

        new classname[8]
        pev(x, pev_classname, classname, charsmax(classname));

        if(equali(classname, "weapon_") || containi(classname, "item_") != -1 || containi(classname, "ammo_") != -1)
            remove_entity(x);
    }
}

public load_ents(const filePath[])
{
	static file, tipo[8], line[100], args[6][9], i, cnt, Float:origin[3], Float:angles[3];
	const aLen = charsmax(args[]); // Esto para que la linea de parse sea "mas corta" :P
	if ((file = fopen(filePath, "r")))
	{
		cnt = 0;
		while (!feof(file))
		{
			fgets(file, line, charsmax(line));
			trim(line);

			if (line[0] && line[0] != ';' && parse(line, tipo, charsmax(tipo), args[0], aLen, args[1], aLen, args[2], aLen, args[3], aLen, args[4], aLen, args[5], aLen) == sizeof(args)+1)
			{
				for (i = 0; i < sizeof(args); i++)
				{
					if (i < 3) origin[i] = str_to_float(args[i]);
					else angles[(i - 3)] = str_to_float(args[i]);
				}
				
				spawn_system_entity((equali(tipo, "snake")) ? 2 : 1, origin, angles);

				cnt++;
			}
		}
		server_print("[SNAKE VS MONKEY] %i loaded entities from ^"%s^"", cnt, filePath);
		fclose(file);
		return 1;
	}
	else log_amx("ERROR: Can't Open Save File for Reading.");
	return 0;
}

/**
 *     STOCKS
**/

stock music_play(track[32])
{
    new iPlayers[32], iPnum;
    get_players(iPlayers, iPnum);

    for(new i = 0; i < iPnum; i++)
    {
        client_cmd(iPlayers[i], "mp3 stop");
        client_cmd(iPlayers[i], "mp3 play %s", track);
    }
}

stock find_element_in_array(array, target)
{
    for(new x = 0;x < sizeof(array);x++)
    {
        if(array[x] == target)
            return array[x];
    }
    
    return -1;
}

stock push_element_to_array(array[MAX_KEROTANS], elem)
{
    for(new x = 0;x < sizeof(array);x++)
    {
        if(array[x] == 0)
        {
            array[x] = elem;
            break;
        }
    }
    
    return array;
}

stock get_filename(mapname[])
{
	formatex(filename, charsmax(filename), "maps/%s_svm.txt", mapname);
}

// this will clean entities from previous matchs
stock clear_field() 
{
	for (new i; i < sizeof g_aszClearFieldEntsClass; i++)
		remove_entity_name(g_aszClearFieldEntsClass[i]);

	new entid;
	while ((entid = find_ent_by_class(entid, "rpg_rocket")))
		set_pev(entid, pev_dmg, 0);

	entid = 0;
	while ((entid = find_ent_by_class(entid, "grenade")))
		set_pev(entid, pev_dmg, 0);
}

stock clear_corpses()
{
	new ent;
	while ((ent = find_ent_by_class(ent, "bodyque")))
		entity_set_origin(ent, Float:{4096.0, 4096.0, 4096.0});
}